#include "bgcollision.h"

#include "map.h"
#include "sprite.h"

/*
  A----B
  |    |
  |    |
  C----D
  C2---D2 (1px below C and D, used to check if sprite is standing on a tile)
  
  Sprites with large collision boxes (>16px) could move through tiles, TODO: Add and check intermediary points for them
*/


#define POINT_A 0
#define POINT_B 1
#define POINT_C 2
#define POINT_D 3
#define POINT_C2 4
#define POINT_D2 5

int test_point(int s, int p)
{
	int x;
	int y;
	switch(p)
	{
		case POINT_A:
			x = spr[s].x;
			y = spr[s].y;
			break;
		case POINT_B:
			x = spr[s].x + spr[s].w;
			y = spr[s].y;
			break;
		case POINT_C:
			x = spr[s].x;
			y = spr[s].y + spr[s].h;
			break;
		case POINT_D:
			x = spr[s].x + spr[s].w;
			y = spr[s].y + spr[s].h;
			break;
		case POINT_C2:
			x = spr[s].x;
			y = spr[s].y + spr[s].h+1;
			break;
		case POINT_D2:
			x = spr[s].x + spr[s].w;
			y = spr[s].y + spr[s].h+1;
			break;
	}
	x /= 16;
	y /= 16;
	return map_is_solid(x,y);
}

void eject_down(int s)
{
	spr[s].y = (((int)spr[s].y+16) & ~15);
	spr[s].vy = 0;
}
void eject_up(int s)
{
	spr[s].y = (((int)spr[s].y+spr[s].h) & ~15) - spr[s].h-1;
	spr[s].vy = 0;
}

void eject_right(int s)
{
	spr[s].x = (((int)spr[s].x+16) & ~15);
	spr[s].vx = 0;
}
void eject_left(int s)
{
	spr[s].x = (((int)spr[s].x+spr[s].w) & ~15) - spr[s].w-1;
	spr[s].vx = 0;
}

void bgcollision_vert(int s)
{
	float vy = spr[s].vy;
	if(spr[s].platform != -1) {
		vy += spr[spr[s].platform].vy;
	}
	else { //no need to do the following check when traveling on a moving platform
		//if ground flag is set, check if we're still standing on a tile
		if(spr[s].ground_flag) {
			if(!test_point(s,POINT_C2) && !test_point(s,POINT_D2)) {
				//we're not, clear ground flag
				spr[s].ground_flag = 0;
			}
		}
	}
	
	if(vy < 0) { //up
		if(test_point(s,POINT_A) || test_point(s,POINT_B))
			eject_down(s);
	}
	else if(!spr[s].ground_flag) { //down or no vertical movement at all
		if(test_point(s,POINT_C) || test_point(s,POINT_D)) {
			eject_up(s);
			spr[s].ground_flag = 1;
		}
	}
	
}

void bgcollision_horz(int s)
{
	float vx = spr[s].vx;
	if(spr[s].platform != -1)
		vx += spr[spr[s].platform].vx;
	if(vx < 0) { //left
		if(test_point(s,POINT_A) || test_point(s,POINT_C))
			eject_right(s);
	}
	else { //right or no horizontal movement at all
		if(test_point(s,POINT_B) || test_point(s,POINT_D))
			eject_left(s);
	}
}



