
#include "sprite.h"

#include <string.h>
#include "map.h"
#include "bgcollision.h"
#include "sprcollision.h"



void sprites_init()
{
	memset(spr,0,sizeof(spr));
	//create player sprite
	spr[0].a = 1;
	spr[0].x = 120;
	spr[0].y = 100;
	spr[0].w = 10;
	spr[0].h = 10;
	spr[0].gravity_flag = 1;
	spr[0].platform = -1;
	
	//platform 1 (movement being controlled from main.c)
	spr[1].a = 1;
	spr[1].x = 90;
	spr[1].y = 148;
	spr[1].w = 32;
	spr[1].h = 8;
	spr[1].platform = -1;
	
	//platform 2
	spr[2].a = 1;
	spr[2].x = 210;
	spr[2].y = 100;
	spr[2].w = 16;
	spr[2].h = 16;
	spr[2].platform = -1;
	
	//platform 3
	spr[3].a = 1;
	spr[3].x = 30;
	spr[3].y = 70;
	spr[3].w = 24;
	spr[3].h = 8;
	spr[3].platform = -1;
}

void sprites_update()
{
	int i;
	//for(i=0;i<MAX_SPR;i++) {
	for(i=MAX_SPR-1;i>=0;i--) { //platforms have to be updated first or else sprite-on-platform movement will look choppy
		if(spr[i].a) {
			int p = spr[i].platform;
			
			//speed limit
				if(spr[i].vx>2)
					spr[i].vx = 2;
				else if(spr[i].vx<-2)
					spr[i].vx = -2;
				if(spr[i].vy>2)
					spr[i].vy = 2;
				else if(spr[i].vy<-2)
					spr[i].vy = -2;
				
			//ground/platform friction
				if(spr[i].ground_flag) {
					if(spr[i].vx > 0.25f)
						spr[i].vx -= 0.25f;
					else if(spr[i].vx < -0.25f)
						spr[i].vx += 0.25f;
					else
						spr[i].vx = 0.0f;
				}
				
			//movement
				int old_x = spr[i].x;
				spr[i].x += spr[i].vx;
				if(p!=-1) { //sprite standing on platform?
					spr[i].x += spr[p].xmov; //add actual pixels the platform moved to the sprite's x position
				}
				bgcollision_horz(i); //check for background collision with walls
				spr[i].xmov = (int)spr[i].x - old_x; //save pixels that the sprite moved horizontally
				
				spr[i].old_y = spr[i].y;
				spr[i].y += spr[i].vy;
				if(p!=-1) {
					spr[i].y = spr[p].y - spr[i].h; //fix vertical position (sprite on platform)
				}
				bgcollision_vert(i); //check for background collision with ground/ceiling
				
			//gravity
				if(spr[i].gravity_flag && !spr[i].ground_flag)
					spr[i].vy += 0.15625f;
		}
	}
	
	//check for collisions between player sprite and platforms
	spr[0].platform = -1;
	spr[0].collision_flag = 0;
	for(i=1;i<MAX_SPR;i++) {
		spr[i].collision_flag = 0;
		if(sprcollision_check(0,i)) {
			spr[0].collision_flag = 1;
			spr[i].collision_flag = 1;
		}
	}
}




